@@ -1,23 +1,22 @@
-def register(*models, **kwargs):
+def register(*models, site=None):
     """
-    Registers the given model(s) classes and wrapped ModelAdmin class with
+    Register the given model(s) classes and wrapped ModelAdmin class with
     admin site:
 
     @register(Author)
     class AuthorAdmin(admin.ModelAdmin):
         pass
 
-    A kwarg of `site` can be passed as the admin site, otherwise the default
-    admin site will be used.
+    The `site` kwarg is an admin site to use instead of the default admin site.
     """
     from django.contrib.admin import ModelAdmin
-    from django.contrib.admin.sites import site, AdminSite
+    from django.contrib.admin.sites import site as default_site, AdminSite
 
     def _model_admin_wrapper(admin_class):
         if not models:
             raise ValueError('At least one model must be passed to register.')
 
-        admin_site = kwargs.pop('site', site)
+        admin_site = site or default_site
 
         if not isinstance(admin_site, AdminSite):
             raise ValueError('site must subclass AdminSite')
