@@ -27,8 +27,9 @@ class ListFilter:
         self.used_parameters = {}
         if self.title is None:
             raise ImproperlyConfigured(
-                "The list filter '%s' does not specify "
-                "a 'title'." % self.__class__.__name__)
+                "The list filter '%s' does not specify a 'title'."
+                % self.__class__.__name__
+            )
 
     def has_output(self):
         """
@@ -66,8 +67,9 @@ class SimpleListFilter(ListFilter):
         super().__init__(request, params, model, model_admin)
         if self.parameter_name is None:
             raise ImproperlyConfigured(
-                "The list filter '%s' does not specify "
-                "a 'parameter_name'." % self.__class__.__name__)
+                "The list filter '%s' does not specify a 'parameter_name'."
+                % self.__class__.__name__
+            )
         if self.parameter_name in params:
             value = params.pop(self.parameter_name)
             self.used_parameters[self.parameter_name] = value
@@ -93,7 +95,8 @@ class SimpleListFilter(ListFilter):
         """
         raise NotImplementedError(
             'The SimpleListFilter.lookups() method must be overridden to '
-            'return a list of tuples (value, verbose value)')
+            'return a list of tuples (value, verbose value).'
+        )
 
     def expected_parameters(self):
         return [self.parameter_name]
@@ -101,13 +104,13 @@ class SimpleListFilter(ListFilter):
     def choices(self, changelist):
         yield {
             'selected': self.value() is None,
-            'query_string': changelist.get_query_string({}, [self.parameter_name]),
+            'query_string': changelist.get_query_string(remove=[self.parameter_name]),
             'display': _('All'),
         }
         for lookup, title in self.lookup_choices:
             yield {
                 'selected': self.value() == str(lookup),
-                'query_string': changelist.get_query_string({self.parameter_name: lookup}, []),
+                'query_string': changelist.get_query_string({self.parameter_name: lookup}),
                 'display': title,
             }
 
@@ -152,9 +155,8 @@ class FieldListFilter(ListFilter):
     @classmethod
     def create(cls, field, request, params, model, model_admin, field_path):
         for test, list_filter_class in cls._field_list_filters:
-            if not test(field):
-                continue
-            return list_filter_class(field, request, params, model, model_admin, field_path=field_path)
+            if test(field):
+                return list_filter_class(field, request, params, model, model_admin, field_path=field_path)
 
 
 class RelatedFieldListFilter(FieldListFilter):
@@ -162,8 +164,8 @@ class RelatedFieldListFilter(FieldListFilter):
         other_model = get_model_from_relation(field)
         self.lookup_kwarg = '%s__%s__exact' % (field_path, field.target_field.name)
         self.lookup_kwarg_isnull = '%s__isnull' % field_path
-        self.lookup_val = request.GET.get(self.lookup_kwarg)
-        self.lookup_val_isnull = request.GET.get(self.lookup_kwarg_isnull)
+        self.lookup_val = params.get(self.lookup_kwarg)
+        self.lookup_val_isnull = params.get(self.lookup_kwarg_isnull)
         super().__init__(field, request, params, model, model_admin, field_path)
         self.lookup_choices = self.field_choices(field, request, model_admin)
         if hasattr(field, 'verbose_name'):
@@ -197,26 +199,19 @@ class RelatedFieldListFilter(FieldListFilter):
     def choices(self, changelist):
         yield {
             'selected': self.lookup_val is None and not self.lookup_val_isnull,
-            'query_string': changelist.get_query_string(
-                {},
-                [self.lookup_kwarg, self.lookup_kwarg_isnull]
-            ),
+            'query_string': changelist.get_query_string(remove=[self.lookup_kwarg, self.lookup_kwarg_isnull]),
             'display': _('All'),
         }
         for pk_val, val in self.lookup_choices:
             yield {
                 'selected': self.lookup_val == str(pk_val),
-                'query_string': changelist.get_query_string({
-                    self.lookup_kwarg: pk_val,
-                }, [self.lookup_kwarg_isnull]),
+                'query_string': changelist.get_query_string({self.lookup_kwarg: pk_val}, [self.lookup_kwarg_isnull]),
                 'display': val,
             }
         if self.include_empty_choice:
             yield {
                 'selected': bool(self.lookup_val_isnull),
-                'query_string': changelist.get_query_string({
-                    self.lookup_kwarg_isnull: 'True',
-                }, [self.lookup_kwarg]),
+                'query_string': changelist.get_query_string({self.lookup_kwarg_isnull: 'True'}, [self.lookup_kwarg]),
                 'display': self.empty_value_display,
             }
 
@@ -228,8 +223,8 @@ class BooleanFieldListFilter(FieldListFilter):
     def __init__(self, field, request, params, model, model_admin, field_path):
         self.lookup_kwarg = '%s__exact' % field_path
         self.lookup_kwarg2 = '%s__isnull' % field_path
-        self.lookup_val = request.GET.get(self.lookup_kwarg)
-        self.lookup_val2 = request.GET.get(self.lookup_kwarg2)
+        self.lookup_val = params.get(self.lookup_kwarg)
+        self.lookup_val2 = params.get(self.lookup_kwarg2)
         super().__init__(field, request, params, model, model_admin, field_path)
         if (self.used_parameters and self.lookup_kwarg in self.used_parameters and
                 self.used_parameters[self.lookup_kwarg] in ('1', '0')):
@@ -245,33 +240,26 @@ class BooleanFieldListFilter(FieldListFilter):
                 ('0', _('No'))):
             yield {
                 'selected': self.lookup_val == lookup and not self.lookup_val2,
-                'query_string': changelist.get_query_string({
-                    self.lookup_kwarg: lookup,
-                }, [self.lookup_kwarg2]),
+                'query_string': changelist.get_query_string({self.lookup_kwarg: lookup}, [self.lookup_kwarg2]),
                 'display': title,
             }
-        if isinstance(self.field, models.NullBooleanField):
+        if self.field.null:
             yield {
                 'selected': self.lookup_val2 == 'True',
-                'query_string': changelist.get_query_string({
-                    self.lookup_kwarg2: 'True',
-                }, [self.lookup_kwarg]),
+                'query_string': changelist.get_query_string({self.lookup_kwarg2: 'True'}, [self.lookup_kwarg]),
                 'display': _('Unknown'),
             }
 
 
-FieldListFilter.register(
-    lambda f: isinstance(f, (models.BooleanField, models.NullBooleanField)),
-    BooleanFieldListFilter
-)
+FieldListFilter.register(lambda f: isinstance(f, models.BooleanField), BooleanFieldListFilter)
 
 
 class ChoicesFieldListFilter(FieldListFilter):
     def __init__(self, field, request, params, model, model_admin, field_path):
         self.lookup_kwarg = '%s__exact' % field_path
         self.lookup_kwarg_isnull = '%s__isnull' % field_path
-        self.lookup_val = request.GET.get(self.lookup_kwarg)
-        self.lookup_val_isnull = request.GET.get(self.lookup_kwarg_isnull)
+        self.lookup_val = params.get(self.lookup_kwarg)
+        self.lookup_val_isnull = params.get(self.lookup_kwarg_isnull)
         super().__init__(field, request, params, model, model_admin, field_path)
 
     def expected_parameters(self):
@@ -280,9 +268,7 @@ class ChoicesFieldListFilter(FieldListFilter):
     def choices(self, changelist):
         yield {
             'selected': self.lookup_val is None,
-            'query_string': changelist.get_query_string(
-                {}, [self.lookup_kwarg, self.lookup_kwarg_isnull]
-            ),
+            'query_string': changelist.get_query_string(remove=[self.lookup_kwarg, self.lookup_kwarg_isnull]),
             'display': _('All')
         }
         none_title = ''
@@ -292,17 +278,13 @@ class ChoicesFieldListFilter(FieldListFilter):
                 continue
             yield {
                 'selected': str(lookup) == self.lookup_val,
-                'query_string': changelist.get_query_string(
-                    {self.lookup_kwarg: lookup}, [self.lookup_kwarg_isnull]
-                ),
+                'query_string': changelist.get_query_string({self.lookup_kwarg: lookup}, [self.lookup_kwarg_isnull]),
                 'display': title,
             }
         if none_title:
             yield {
                 'selected': bool(self.lookup_val_isnull),
-                'query_string': changelist.get_query_string({
-                    self.lookup_kwarg_isnull: 'True',
-                }, [self.lookup_kwarg]),
+                'query_string': changelist.get_query_string({self.lookup_kwarg_isnull: 'True'}, [self.lookup_kwarg]),
                 'display': none_title,
             }
 
@@ -387,8 +369,8 @@ class AllValuesFieldListFilter(FieldListFilter):
     def __init__(self, field, request, params, model, model_admin, field_path):
         self.lookup_kwarg = field_path
         self.lookup_kwarg_isnull = '%s__isnull' % field_path
-        self.lookup_val = request.GET.get(self.lookup_kwarg)
-        self.lookup_val_isnull = request.GET.get(self.lookup_kwarg_isnull)
+        self.lookup_val = params.get(self.lookup_kwarg)
+        self.lookup_val_isnull = params.get(self.lookup_kwarg_isnull)
         self.empty_value_display = model_admin.get_empty_value_display()
         parent_model, reverse_path = reverse_field_path(model, field_path)
         # Obey parent ModelAdmin queryset when deciding which options to show
@@ -396,10 +378,7 @@ class AllValuesFieldListFilter(FieldListFilter):
             queryset = model_admin.get_queryset(request)
         else:
             queryset = parent_model._default_manager.all()
-        self.lookup_choices = (queryset
-                               .distinct()
-                               .order_by(field.name)
-                               .values_list(field.name, flat=True))
+        self.lookup_choices = queryset.distinct().order_by(field.name).values_list(field.name, flat=True)
         super().__init__(field, request, params, model, model_admin, field_path)
 
     def expected_parameters(self):
@@ -408,7 +387,7 @@ class AllValuesFieldListFilter(FieldListFilter):
     def choices(self, changelist):
         yield {
             'selected': self.lookup_val is None and self.lookup_val_isnull is None,
-            'query_string': changelist.get_query_string({}, [self.lookup_kwarg, self.lookup_kwarg_isnull]),
+            'query_string': changelist.get_query_string(remove=[self.lookup_kwarg, self.lookup_kwarg_isnull]),
             'display': _('All'),
         }
         include_none = False
@@ -419,17 +398,13 @@ class AllValuesFieldListFilter(FieldListFilter):
             val = str(val)
             yield {
                 'selected': self.lookup_val == val,
-                'query_string': changelist.get_query_string({
-                    self.lookup_kwarg: val,
-                }, [self.lookup_kwarg_isnull]),
+                'query_string': changelist.get_query_string({self.lookup_kwarg: val}, [self.lookup_kwarg_isnull]),
                 'display': val,
             }
         if include_none:
             yield {
                 'selected': bool(self.lookup_val_isnull),
-                'query_string': changelist.get_query_string({
-                    self.lookup_kwarg_isnull: 'True',
-                }, [self.lookup_kwarg]),
+                'query_string': changelist.get_query_string({self.lookup_kwarg_isnull: 'True'}, [self.lookup_kwarg]),
                 'display': self.empty_value_display,
             }
 
