@@ -1,12 +1,9 @@
 import datetime
-import warnings
 
 from django.forms.utils import flatatt, pretty_name
 from django.forms.widgets import Textarea, TextInput
-from django.utils.deprecation import RemovedInDjango21Warning
 from django.utils.functional import cached_property
 from django.utils.html import conditional_escape, format_html, html_safe
-from django.utils.inspect import func_accepts_kwargs, func_supports_parameter
 from django.utils.safestring import mark_safe
 from django.utils.translation import gettext_lazy as _
 
@@ -82,40 +79,18 @@ class BoundField:
         attributes passed as attrs. If a widget isn't specified, use the
         field's default widget.
         """
-        if not widget:
-            widget = self.field.widget
-
+        widget = widget or self.field.widget
         if self.field.localize:
             widget.is_localized = True
-
         attrs = attrs or {}
         attrs = self.build_widget_attrs(attrs, widget)
-        auto_id = self.auto_id
-        if auto_id and 'id' not in attrs and 'id' not in widget.attrs:
-            if not only_initial:
-                attrs['id'] = auto_id
-            else:
-                attrs['id'] = self.html_initial_id
-
-        if not only_initial:
-            name = self.html_name
-        else:
-            name = self.html_initial_name
-
-        kwargs = {}
-        if func_supports_parameter(widget.render, 'renderer') or func_accepts_kwargs(widget.render):
-            kwargs['renderer'] = self.form.renderer
-        else:
-            warnings.warn(
-                'Add the `renderer` argument to the render() method of %s. '
-                'It will be mandatory in Django 2.1.' % widget.__class__,
-                RemovedInDjango21Warning, stacklevel=2,
-            )
+        if self.auto_id and 'id' not in widget.attrs:
+            attrs.setdefault('id', self.html_initial_id if only_initial else self.auto_id)
         return widget.render(
-            name=name,
+            name=self.html_initial_name if only_initial else self.html_name,
             value=self.value(),
             attrs=attrs,
-            **kwargs
+            renderer=self.form.renderer,
         )
 
     def as_text(self, attrs=None, **kwargs):
@@ -175,7 +150,7 @@ class BoundField:
         if id_:
             id_for_label = widget.id_for_label(id_)
             if id_for_label:
-                attrs = dict(attrs or {}, **{'for': id_for_label})
+                attrs = {**(attrs or {}), 'for': id_for_label}
             if self.field.required and hasattr(self.form, 'required_css_class'):
                 attrs = attrs or {}
                 if 'class' in attrs:
@@ -241,8 +216,7 @@ class BoundField:
         return data
 
     def build_widget_attrs(self, attrs, widget=None):
-        if not widget:
-            widget = self.field.widget
+        widget = widget or self.field.widget
         attrs = dict(attrs)  # Copy attrs to avoid modifying the argument.
         if widget.use_required_attribute(self.initial) and self.field.required and self.form.use_required_attribute:
             attrs['required'] = True
@@ -274,7 +248,7 @@ class BoundWidget:
         return self.tag(wrap_label=True)
 
     def tag(self, wrap_label=False):
-        context = {'widget': self.data, 'wrap_label': wrap_label}
+        context = {'widget': {**self.data, 'wrap_label': wrap_label}}
         return self.parent_widget._render(self.template_name, context, self.renderer)
 
     @property
