@@ -19,7 +19,7 @@ def unlocalize(value):
     Force a value to be rendered as a non-localized value,
     regardless of the value of ``settings.USE_L10N``.
     """
-    return str(value)
+    return str(formats.localize(value, use_l10n=False))
 
 
 class LocalizeNode(Node):
